/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  GThumb
 *
 *  Copyright (C) 2010 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <gthumb.h>

G_BEGIN_DECLS

#define PREF_CONTACT_SHEET_HEADER                   "/apps/gthumb/ext/contact_sheet/header"
#define PREF_CONTACT_SHEET_FOOTER                   "/apps/gthumb/ext/contact_sheet/footer"
#define PREF_CONTACT_SHEET_DESTINATION              "/apps/gthumb/ext/contact_sheet/destination"
#define PREF_CONTACT_SHEET_TEMPLATE                 "/apps/gthumb/ext/contact_sheet/template"
#define PREF_CONTACT_SHEET_MIME_TYPE                "/apps/gthumb/ext/contact_sheet/mime_type"
#define PREF_CONTACT_SHEET_HTML_IMAGE_MAP           "/apps/gthumb/ext/contact_sheet/html_image_map"
#define PREF_CONTACT_SHEET_THEME                    "/apps/gthumb/ext/contact_sheet/theme"
#define PREF_CONTACT_SHEET_IMAGES_PER_PAGE          "/apps/gthumb/ext/contact_sheet/images_per_page"
#define PREF_CONTACT_SHEET_SINGLE_PAGE              "/apps/gthumb/ext/contact_sheet/single_page"
#define PREF_CONTACT_SHEET_COLUMNS                  "/apps/gthumb/ext/contact_sheet/columns"
#define PREF_CONTACT_SHEET_SORT_TYPE                "/apps/gthumb/ext/contact_sheet/sort_type"
#define PREF_CONTACT_SHEET_SORT_INVERSE             "/apps/gthumb/ext/contact_sheet/sort_inverse"
#define PREF_CONTACT_SHEET_SAME_SIZE                "/apps/gthumb/ext/contact_sheet/same_size"
#define PREF_CONTACT_SHEET_THUMBNAIL_SIZE           "/apps/gthumb/ext/contact_sheet/thumbnail_size"
#define PREF_CONTACT_SHEET_SQUARED_THUMBNAIL        "/apps/gthumb/ext/contact_sheet/squared_thumbnail"
#define PREF_CONTACT_SHEET_THUMBNAIL_CAPTION        "/apps/gthumb/ext/contact_sheet/thumbnail_caption"

#define PREF_IMAGE_WALL_DESTINATION                 "/apps/gthumb/ext/image_wall/destination"
#define PREF_IMAGE_WALL_TEMPLATE                    "/apps/gthumb/ext/image_wall/template"
#define PREF_IMAGE_WALL_MIME_TYPE                   "/apps/gthumb/ext/image_wall/mime_type"
#define PREF_IMAGE_WALL_IMAGES_PER_PAGE             "/apps/gthumb/ext/image_wall/images_per_page"
#define PREF_IMAGE_WALL_SINGLE_PAGE                 "/apps/gthumb/ext/image_wall/single_page"
#define PREF_IMAGE_WALL_COLUMNS                     "/apps/gthumb/ext/image_wall/columns"
#define PREF_IMAGE_WALL_SORT_TYPE                   "/apps/gthumb/ext/image_wall/sort_type"
#define PREF_IMAGE_WALL_SORT_INVERSE                "/apps/gthumb/ext/image_wall/sort_inverse"
#define PREF_IMAGE_WALL_THUMBNAIL_SIZE              "/apps/gthumb/ext/image_wall/thumbnail_size"

G_END_DECLS

#endif /* PREFERENCES_H */
