/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  GThumb
 *
 *  Copyright (C) 2011 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <gthumb.h>

G_BEGIN_DECLS

#define  PREF_CHANGE_DATE_SET_LAST_MODIFIED_DATE          "/apps/gthumb/ext/change_date/set_last_modified_date"
#define  PREF_CHANGE_DATE_SET_COMMENT_DATE		  "/apps/gthumb/ext/change_date/set_comment_date"
#define  PREF_CHANGE_DATE_SET_EXIF_DATETIMEORIGINAL_TAG   "/apps/gthumb/ext/change_date/set_exif_datetimeoriginal_tag"
#define  PREF_CHANGE_DATE_TO_FOLLOWING_DATE               "/apps/gthumb/ext/change_date/to_following_date"
#define  PREF_CHANGE_DATE_DATE                            "/apps/gthumb/ext/change_date/date"
#define  PREF_CHANGE_DATE_TO_FILE_MODIFIED_DATE           "/apps/gthumb/ext/change_date/to_file_modified_date"
#define  PREF_CHANGE_DATE_TO_FILE_CREATION_DATE           "/apps/gthumb/ext/change_date/to_file_creation_date"
#define  PREF_CHANGE_DATE_TO_PHOTO_ORIGINAL_DATE          "/apps/gthumb/ext/change_date/to_photo_original_date"
#define  PREF_CHANGE_DATE_ADJUST_TIME                     "/apps/gthumb/ext/change_date/adjust_time"
#define  PREF_CHANGE_DATE_TIME_ADJUSTMENT                 "/apps/gthumb/ext/change_date/time_adjustment"

G_END_DECLS

#endif /* PREFERENCES_H */
